import { startup, updateLifeCycle, changeEditorState, transformState, configureToolbar, linkPromptChangeState, configureEditorApi, init, updateWordCount, updateCharCount, throwConfigurationError, globalApiConfig, toggleAlignment } from './creators';
import Types from './types';
import expect from 'expect';

describe('action creators tests', ()=>{
  it('startup action should return relevant payload',()=>{
    const actionResult = startup({ test: true }, []);
    expect(actionResult).toIncludeKeys(['config', 'type', 'editorState', 'plugins']);
    expect(actionResult.config.test).toBeTruthy();
    expect(actionResult.type).toEqual(Types.CONFIGURE_EDITOR);
  });

  it('updateLifeCycle action should return relevant payload',()=>{
    const actionResult = updateLifeCycle('configured', {});
    expect(actionResult).toIncludeKeys(['type', 'lifecycle']);
    expect(actionResult.lifecycle).toEqual('configured');
    expect(actionResult.type).toEqual(Types.UPDATE_LIFECYCLE);
  });

  it('changeEditorState action should return relevant payload',()=>{
    const actionResult = changeEditorState({}, {}, 'unstyled');
    expect(actionResult).toIncludeKeys(['editorState', 'type', 'activeEntityAtCursor']);
    expect(actionResult.activeEntityAtCursor).toEqual('unstyled');
    expect(actionResult.type).toEqual(Types.EDITOR_CHANGE_STATE);
  });

  it('transformState action should return relevant payload',()=>{
    const actionResult = transformState({ test: true });
    expect(actionResult).toIncludeKeys(['json', 'type']);
    expect(actionResult.json.test).toBeTruthy();
    expect(actionResult.type).toEqual(Types.EDITOR_TRANSFORM_TO_RAW_STATE);
  });


  it('configureToolbar action should return relevant payload',()=>{
    const actionResult = configureToolbar({ test: true });
    expect(actionResult).toIncludeKeys(['toolbarConfig', 'type']);
    expect(actionResult.toolbarConfig.test).toBeTruthy();
    expect(actionResult.type).toEqual(Types.CONFIGURE_TOOLBAR);
  });

  it('linkPromptChangeState action should return relevant payload',()=>{
    const actionResult = linkPromptChangeState({ test: true });
    expect(actionResult).toIncludeKeys(['linkPrompt', 'type']);
    expect(actionResult.linkPrompt.test).toBeTruthy();
    expect(actionResult.type).toEqual(Types.LINK_CHANGE_STATE);
  });

  it('configureEditorApi action should return relevant payload',()=>{
    const actionResult = configureEditorApi({ test: true });
    expect(actionResult).toIncludeKeys(['subscribers', 'type']);
    expect(actionResult.subscribers.test).toBeTruthy();
    expect(actionResult.type).toEqual(Types.CONFIGURE_EDITOR_API);
  });

  it('init action should return relevant payload',()=>{
    const actionResult = init();
    expect(actionResult).toIncludeKeys(['type']);
    expect(actionResult.type).toEqual(Types.MOUNT_EDITOR)
  });

  it('updateWordCount action should return relevant payload',()=>{
    const actionResult = updateWordCount(3);
    expect(actionResult).toIncludeKeys(['type', 'wordCount']);
    expect(actionResult.wordCount).toEqual(3);
    expect(actionResult.type).toEqual(Types.UPDATE_WORD_COUNT);
  });

  it('updateCharCount action should return relevant payload',()=>{
    const actionResult = updateCharCount(2);
    expect(actionResult).toIncludeKeys(['type','charCount']);
    expect(actionResult.charCount).toEqual(2);
    expect(actionResult.type).toEqual(Types.UPDATE_CHAR_COUNT);
  });

  it('throwConfigurationError action should return relevant payload',()=>{
    const actionResult = throwConfigurationError("Error");
    expect(actionResult).toIncludeKeys(['type','message']);
    expect(actionResult.message).toEqual("Error");
    expect(actionResult.type).toEqual(Types.CONFIGURATION_ERROR);
  });

  it('globalApiConfig action should return relevant payload',()=>{
    const actionResult = globalApiConfig({ test: true });
    expect(actionResult).toIncludeKeys(['type','apiConfig']);
    expect(actionResult.apiConfig.test).toBeTruthy();
    expect(actionResult.type).toEqual(Types.GLOBAL_API_CONFIG);
  });

  it('toggleAlignment action should return relevant payload',()=>{
    const actionResult = toggleAlignment('right');
    expect(actionResult).toIncludeKeys(['type','alignment']);
    expect(actionResult.alignment).toEqual('right');
    expect(actionResult.type).toEqual(Types.TOGGLE_ALIGNMENT);
  });

});
