// @flow
import React from 'react';
import classNames from '../../utils/classNames';

// DraftJS Specific
import { Editor, ContentBlock } from 'draft-js';
//Styling
import styles from './editorArea.module.scss';


export function blockStyleFn(block: ContentBlock){
  const type = block.getType();
  const data = block.getData();
  const blockTypeClassName = styles[`block__${ type }`];
  const alignmentValue = !data.isEmpty() && data.get('alignment') || 'left';
  const alignmentClassName = styles[`block__alignment--${ alignmentValue }`];
  return [ blockTypeClassName, alignmentClassName ].join(' ');
}

export default class EditorArea extends React.Component {
  editor: Function;


  render(){
    const { editorState, changeState, blockRenderMap, onTab, blockRendererFn, handleKeyCommand, handleReturn } = this.props;
    return (
      <div className={ classNames(styles.container) }>
        <Editor ref={ (editor) => { this.editor = editor; } }
          editorState={ editorState }
          onChange={ changeState }
          blockStyleFn={ blockStyleFn }
          blockRenderMap={ blockRenderMap }
          onTab={ onTab }
          blockRendererFn={ blockRendererFn }
          handleKeyCommand={ handleKeyCommand }
          handleReturn={ handleReturn }
        />
      </div>
    );
  }
}