import React from 'react';
import { shallow } from 'enzyme';
import EditorArea, { blockStyleFn } from './editorArea';
import styles from './editorArea.module.scss';
import { Editor, ContentBlock } from 'draft-js';
import { fromJS } from 'immutable';
import expectJS from 'expect';


describe('editorArea and customizations tests', () => {
  let shallowArea;
  beforeEach(()=>{
    shallowArea = shallow(<EditorArea/>);
  });
  it('should render editorArea with proper className', () => {
    expect(shallowArea.props().className).toBe(styles.container);
  });
  it('should include draft-js Editor', () => {
    expect(shallowArea.find(Editor).length).toBe(1);
  });

  it('should pass correct props to DraftJS', () => {
    expectJS(shallowArea.find(Editor).props()).toIncludeKeys([
      'editorState',
      'onChange',
      'blockStyleFn',
      'blockRenderMap',
      'onTab',
      'blockRendererFn',
      'handleKeyCommand',
      'handleReturn'
    ]);
  });

  describe('blockStyleFn tests', () => {
    it('should pass correct classname on block without alignment', () => {
      const block = new ContentBlock({
        key: 'asdad',
        type: 'unstyled',
        text: 'asdasda',
        data: fromJS({
          alignment: 'right'
        })
      });
      expect(blockStyleFn(block)).toBe([styles['block__unstyled'], styles[`block__alignment--right `]].join(' '))
    });
  });
});
