import React from 'react';
import { shallow } from 'enzyme';
import { EditorComponent,  blockRenderMap, blockRendererFn } from './editorComponent';
import Toolbar from '../toolbar/toolbar';
import mockedInitialState from '../../../src/__mocks__/editorState.json';
import { buttons, linkCaptions } from '../../config/defaults';

import styles from './editorComponent.module.scss';
import { EditorState, ContentBlock, convertFromRaw, CompositeDecorator } from 'draft-js';
import EditorArea from '../editorArea/editorArea';
import Separator from '../separator/separator';
import expectJS from 'expect';

describe('editorComponent tests', () => {
  let component;
  beforeEach(()=>{
    component = shallow(
      <EditorComponent init={ true }
        toolbarConfig={ { ...buttons, linkCaptions, configured: true } }
        linkPrompt={ {
          showLinkPrompt: false,
          urlValue: '',
          linkCaption: ''
        } }
      />
    );
  })
  it('should render editorComponent with proper className', () => {
    expect(component.props().className).toBe(styles.container);
  });
  it('should include EditorArea', () => {
    expect(component.find(EditorArea).length).toBe(1);
  });
  it('should pass needed props to EditorArea', () => {
    expectJS(component.find(EditorArea).props()).toIncludeKeys([
      'editorState',
      'changeState',
      'blockRendererFn',
      'blockRenderMap',
      'handleKeyCommand',
      'handleReturn',
      'onTab'
    ]);
  });

  it('should include Toolbar', () => {
    expect(component.find(Toolbar).length).toBe(1);
  });
  it('should pass needed props to Toolbar', () => {
    expectJS(component.find(Toolbar).props()).toIncludeKeys(['onToggle', 'toolbarConfig', 'editorState', 'toggleAlignment', 'linkToggle']);
  });

  it('should pass correct blockRenderMap', ()=>{
    expectJS(blockRenderMap.toJS()).toIncludeKeys(['hr']);
  });
  it('should return correct separator block when block type is hr', ()=>{
    const block = new ContentBlock({
      key: 'asdad',
      type: 'hr',
      text: 'asdasda'
    });
    expectJS(blockRendererFn(block).component).toExist();
    expectJS(blockRendererFn(block).component).toEqual(Separator);
    expect(blockRendererFn(block).editable).toBeFalsy();
  });
  it('should pass correct blockRenderMap', ()=>{
    expectJS(blockRenderMap.toJS()).toIncludeKeys(['hr']);
  });
  it('handleKeyCommand should return false when common command is triggered', () => {
    const editorState = EditorState.createEmpty();
    const editorComponent = shallow(
      <EditorComponent
        editorState={ editorState}
        toolbarConfig={ { ...buttons, linkCaptions, configured: true } }
        linkPrompt={ {
          showLinkPrompt: false,
          urlValue: '',
          linkCaption: ''
         } }
      />
    );
    expect(editorComponent.instance().handleKeyCommand('split-block')).toBeFalsy();
  });

  // it('onTab should return false when common command is triggered', () => {
  //   const editorState = EditorState.createEmpty();
  //   const editorComponent = shallow(<EditorComponent editorState={ editorState} />);
  //   expect(editorComponent.instance().handleKeyCommand('split-block')).toBeFalsy();
  // });

});