// @flow
import React from 'react';
import './editorWrapper.scss';
import 'draft-js/dist/Draft.css';
import storeGenerator from '../core/storeGenerator';
import { Provider } from 'react-redux';
import EditorComponent from './editorComponent/editorComponent';
import { startup } from '../actions/creators';
import filterProps from '../utils/filterProps';
import { EditorState, CompositeDecorator } from 'draft-js';
import linkPlugin from './link/linkPlugin';

export default class EditorWrapper extends React.Component {

  store: Object;
  plugins: Array<Object>;

  constructor(props :Object){
    super(props);
    this.store = storeGenerator(props.blackList);
    this.plugins = [linkPlugin()];
  }

  componentDidMount(){
    const customPlugins = new CompositeDecorator(this.plugins);
    const editorState = EditorState.createEmpty(customPlugins);
    this.store.dispatch(
      startup(filterProps(this.props, ['blackList']), editorState, this.plugins, this.store)
    );
  }

  render(){
    return (
      <Provider store={ this.store }>
        <EditorComponent { ...this.props }/>
      </Provider>
    );
  }
}