import React from 'react';
import ReactDOM from 'react-dom';
import EditorWrapper from './editorWrapper';
import { shallow } from 'enzyme';
import expectJS from 'expect';
import storeGenerator from '../core/storeGenerator';
import Types from '../actions/types';

const testActionsBlackList = Object.keys(Types);

describe('Public Facade tests', () =>{
  it('renders without crashing', () => {
    const div = document.createElement('div');
    ReactDOM.render(<EditorWrapper blackList={testActionsBlackList}/>, div);
  });

  it('EditorComponent is wrapped in Provider', ()=>{
    const wrapper = shallow(<EditorWrapper/>);
    expect(wrapper.name()).toEqual('Provider');
  });
  it('Provider wrapper gets correct store prop', ()=>{
    const wrapper = shallow(<EditorWrapper/>);
    expectJS(wrapper.props().store).toEqual(storeGenerator());
  });

  it('expect startup action to be called', ()=>{
    let creators = require('../actions/creators');
    //noinspection JSAnnotator
    creators.startup = jest.fn((config, plugins)=>({
      type: 'Test'
    }));
    const div = document.createElement('div');
    ReactDOM.render(<EditorWrapper initialState="" blackList={["Test", ...testActionsBlackList]}/>, div);
    expect(creators.startup).toBeCalled();
  });
});


