import React from 'react';
import autoBind from 'react-autobind';
import { SelectionState, Entity, EditorState, RichUtils } from 'draft-js';
import { getSelectedText } from '../../utils/draftUtils';
import linkClassNames from './linkPlugin.module.scss';

export default class Link extends React.Component {

  state: {
    showEdit: Boolean
  };

  constructor(props){
    super(props);
    this.state = {
      showEdit: false
    };
    autoBind(this);
  }

  componentWillReceiveProps(newProps){
    if (newProps.activeEntityAtCursor === 'NONE'){
      this.setState({
        showEdit: false
      });
    }
  }


  constructSelectedState(block, offset, forward){
    const entityKey = block.getEntityAt(offset);
    if (entityKey && Entity.get(entityKey).getType() === 'LINK'){
      const newOffset = forward ? offset + 1 : offset - 1;
      return this.constructSelectedState(block, newOffset, forward);
    }
    return offset;
  }

  getSelectionForSelectedBlock(editorState){
    const blockKey = this.props.offsetKey.split('-')[0];
    const block = editorState.getCurrentContent().getBlockForKey(blockKey);
    const startOffset = this.constructSelectedState(block, editorState.getSelection().getFocusOffset() - 1, false);
    const endOffset = this.constructSelectedState(block, editorState.getSelection().getFocusOffset() - 1, true);
    const selectionState = SelectionState.createEmpty(blockKey);
    return {
      block,
      blockKey,
      selectionState: selectionState.merge({
        focusOffset: endOffset,
        anchorOffset: startOffset + 1,
        hasFocus: true
      })
    };
  }

  updateSelection(clearLink){
    let { changeEditorState, editorState } = this.props;
    const { selectionState, block } = this.getSelectionForSelectedBlock(editorState);
    editorState = EditorState.forceSelection(editorState, selectionState);
    if (clearLink){
      editorState = RichUtils.toggleLink(EditorState.forceSelection(editorState, selectionState), selectionState, null);
    }
    changeEditorState(
      editorState,
      null,
      block.getEntityAt(editorState.getSelection().getFocusOffset() - 1)
    );
    return getSelectedText(selectionState, editorState);
  }

  editLink(url){
    const { linkPromptChangeState } = this.props;
    const linkCaption = this.updateSelection();
    linkPromptChangeState({
      showLinkPrompt: true,
      urlValue: url,
      linkCaption
    });
  }

  unlink(){
    this.updateSelection(true);
  }

  render(){
    const { entityKey, children, activeEntityAtCursor, editCaption, unlinkCaption } = this.props;
    const { url } = Entity.get(entityKey).getData();
    const {
      linkPrompt__tooltip: linkPromptTooltip
    } = linkClassNames;
    return (
      <span className={ linkClassNames.decorator }>
      <a href={ url } className={ linkClassNames.decorator__link } onClick={ ()=>{ this.setState({ showEdit: true }); } }>
        { children }
      </a>
        { this.state.showEdit && activeEntityAtCursor === 'LINK' ?
          <div className={ linkPromptTooltip }>
            <button onClick={ () => { window.open(url, '_blank'); } }>{ url }</button>
            <button onClick={ (e) => { e.stopPropagation(); this.editLink(url); } }>{ editCaption }</button>
            <button onClick={ (e) => { e.stopPropagation(); this.unlink(); } }>{ unlinkCaption } </button>
          </div>
          : false }
    </span>
    );
  }
}