import React from 'react';
import autoBind from 'react-autobind';
import { Entity, EditorState, Modifier } from 'draft-js';
import linkClassNames from './linkPlugin.module.scss';
import { getSelectedText } from '../../utils/draftUtils';
import { insertBlankSpace } from '../../utils/draftUtils';

export default class LinkPrompt extends React.Component {
  constructor(props){
    super(props);
    this.state = {
      focused: false
    };
    autoBind(this);
  }

  onUrlChange(e){
    this.props.changeState({
      urlValue: e.target.value
    });
  }

  onCaptionChange(e){
    this.props.changeState({
      linkCaption: e.target.value
    });
  }

  onLinkInputKeyDown(e) {
    switch (e.which) {
      case 13:
        this.confirmLink(e);
        break;
      case 27:
        this.cancelLink(e);
        break;
      default:
        break;
    }
  }


  confirmLink(e){
    e.preventDefault();
    const { editorState, urlValue, onConfirm, onClose } = this.props;
    const selection = editorState.getSelection();
    const url = /http(|s):\/\//.test(urlValue) ? urlValue : `http://${urlValue}`;
    const entityKey = Entity.create('LINK', 'MUTABLE', { url });
    const decoratedLinkPayload = [
      editorState.getCurrentContent(),
      selection,
      this.props.linkCaption || getSelectedText(editorState.getSelection(), editorState),
      null,
      entityKey
    ];
    const isInsert = selection.isCollapsed();
    const { insertText, replaceText } = Modifier;
    const textModificationMethod = isInsert ? insertText : replaceText;
    let newEditorState = EditorState.push(editorState, textModificationMethod(...decoratedLinkPayload));
    onConfirm(insertBlankSpace(newEditorState));
    this.closePrompt();
    onClose();
  }

  removeFocusedFlag(){
    this.setState({
      focused: false
    });
  }

  closePrompt(){
    this.removeFocusedFlag();
    this.props.changeState({
      showLinkPrompt: false,
      urlValue: '',
      linkCaption: ''
    });
  }

  cancelLink(e){
    e.preventDefault();
    this.removeFocusedFlag();
    this.closePrompt();
    this.props.onClose();
  }

  render(){
    const { captions: { confirmText, cancelText, title, captionLabelText, urlLabelText }, isOpen, linkCaption, editorState } = this.props;
    if (isOpen) {
      const {
        linkPrompt: linkPromptClassName,
        linkPrompt__overlay: linkPromptOverlayClassName,
        linkPrompt__label: linkPromptLabelClassName,
        linkPrompt__title: linkPromptTitleClassName,
        linkPrompt__input: linkPromptInputClassName,
        linkPrompt__buttonsWrapper: linkPromptButtonsWrapperClassName,
        linkPrompt__button: linkPromptButtonClassName,
        linkPrompt__inputGroup: linkPromptInputGroupClassName,
        linkPrompt__closeX: linkPromptXCloseModalClassName
      } = linkClassNames;
      setImmediate(()=>{
        if (!this.state.focused) {
          const refToFocus = linkCaption || getSelectedText(editorState.getSelection(), editorState) ? 'url' : 'caption';
          this[refToFocus].focus();
          this.setState({
            focused: true
          });
        }
      });
      return (
        <div>
          <div className={ linkPromptOverlayClassName } onClick={ this.cancelLink }></div>
          <div className={ linkPromptClassName }>
            <label className={ linkPromptTitleClassName }>{ title }</label>
            <div className={ linkPromptXCloseModalClassName } onClick={ this.cancelLink }></div>
            <div className={ linkPromptInputGroupClassName }>
              <label className={ linkPromptLabelClassName }>{ urlLabelText }</label>
              <input onChange={ this.onUrlChange }
                ref={ (url) => { this.url = url; } }
                type="text"
                value={ this.props.urlValue }
                className= { linkPromptInputClassName }
                onKeyDown={ this.onLinkInputKeyDown }
              />
            </div>
            <div className={ linkPromptInputGroupClassName }>
              <label className={ linkPromptLabelClassName }>{ captionLabelText }</label>
              <input onChange={ this.onCaptionChange }
                ref={ (caption) => { this.caption = caption; } }
                type="text"
                value={ linkCaption || getSelectedText(editorState.getSelection(), editorState) }
                className= { linkPromptInputClassName }
                onKeyDown={ this.onLinkInputKeyDown }
              />
            </div>
            <div className={ linkPromptButtonsWrapperClassName }>
              <button onClick={ this.confirmLink } className={ linkPromptButtonClassName }>
                { confirmText }
              </button>
              <button onClick={ this.cancelLink } className={ linkPromptButtonClassName }>
                { cancelText }
              </button>
            </div>
          </div>
        </div>
      );
    }
    return false;
  }
}