import React from 'react';
import styles from './toolbar.module.scss';
import { RichUtils, EditorState, } from 'draft-js';
import autoBind from 'react-autobind';
import Button from '../button/button';
import { alignBlock, addBlock } from '../../utils/draftUtils';
export const toolbarClassNames = styles;

export function populateToolbarButtons(config, toggleControls, isActive, isMenuButton){
  const { buttonsList, buttonsConf } = config;
  return buttonsList.map((button) => {
    const bConf = buttonsConf[button];
    const active = isActive(bConf.controlType, bConf.blockType);
    const buttonProps = { ...bConf, active, toggle: toggleControls[`${bConf.controlType}Toggle`], isMenuButton };
    return bConf.controlType !== 'menu' ? <Button key={ button } { ...buttonProps }>{ button }</Button> :
      <div key={ button } className={ toolbarClassNames.menu }>
        <label className={ toolbarClassNames.menuLabel }>{ bConf.label }</label>
        <div className={ toolbarClassNames.menu__container }>
          { populateToolbarButtons({ buttonsConf: bConf.buttons, buttonsList: Object.keys(bConf.buttons) }, toggleControls, isActive, true) }
        </div>
      </div>;
  });
}

class Toolbar extends React.Component {

  constructor(props){
    super(props);
    autoBind(this);
  }


  alignmentToggle(alignment){
    const { editorState, onToggle, toggleAlignment } = this.props;
    let contentState = editorState.getCurrentContent();
    const selectionState = editorState.getSelection();
    contentState = alignBlock(contentState, selectionState, alignment);
    onToggle(EditorState.push(editorState, contentState));
    toggleAlignment(alignment);
  }

  linkToggle(blockType){
    const { linkToggle, editorState, onToggle } = this.props;
    if (blockType === 'remove') {
      const selection = editorState.getSelection();
      if (!selection.isCollapsed()) {
        onToggle(RichUtils.toggleLink(editorState, selection, null));
      }
    }
    if (blockType === 'add') {
      linkToggle({ showLinkPrompt: true });
    }
  }


  customBlockToggle(blockType){
    const { editorState, onToggle } = this.props;
    const selection = editorState.getSelection();
    onToggle(addBlock(editorState, selection, blockType));
  }

  blockToggle(blockType, e){
    const { onToggle, editorState } = this.props;
    onToggle(RichUtils.toggleBlockType(editorState, blockType));
  }
  inlineToggle(blockType, e){
    const { onToggle, editorState } = this.props;
    onToggle(RichUtils.toggleInlineStyle(editorState, blockType));
  }

  isButtonActivePredicate(controlType, blockType) {
    const { editorState } = this.props;
    const currentStyle = editorState.getCurrentInlineStyle();
    const selection = editorState.getSelection();
    const currentBlockType = editorState
      .getCurrentContent()
      .getBlockForKey(selection.getStartKey())
      .getType();
    switch (controlType) {
      case 'inline':
        return currentStyle.has(blockType);
      case 'block':
        return currentBlockType === blockType;
      default:
        return false;
    }
  }

  render(){
    const { container: containerClassName } = toolbarClassNames;
    const { toolbarConfig, children } = this.props;
    const { inlineToggle, alignmentToggle, blockToggle, linkToggle, isButtonActivePredicate, customBlockToggle } = this;
    return (
      toolbarConfig && toolbarConfig.configured ?
        <div className={ containerClassName }>
          { [
            populateToolbarButtons(toolbarConfig, { inlineToggle, alignmentToggle, blockToggle, linkToggle, customBlockToggle }, isButtonActivePredicate),
            children
          ] }
        </div>
        : false
    );
  }
}

export default Toolbar;
