import React from 'react';
import { shallow } from 'enzyme';
import expectJS from 'expect';
import Toolbar, { populateToolbarButtons as toolbarButtonsPopulator } from './toolbar';
import toolbarClassNames from './toolbar.module.scss';
import { EditorComponent } from '../editorComponent/editorComponent';
import { buttons, linkCaptions } from '../../config/defaults';
import { EditorState, RichUtils } from 'draft-js';
import { alignBlock, addBlock } from '../../utils/draftUtils';

const expectedValues = {
  rendering: {
    toolbarClassNames
  }
};

describe('Toolbar tests', ()=>{
  let wrapper;
  let toolbar;
  beforeEach(()=>{
    toolbar = shallow(<Toolbar editorState={ EditorState.createEmpty() } toolbarConfig={ { ...buttons, linkCaptions, configured: true } } />);
  });
  describe('Basic Rendering', ()=>{

    const { toolbarClassNames: { container: containerClassName } } = expectedValues.rendering;
    it('should have correct container className', ()=>{
      expectJS(toolbar.props().className).toEqual(containerClassName);
    });
    it('should have isButtonActivePredicate, inlineToggle, blockToggle, linkToggle, alignmentToggle and customBlockToggle functions', ()=>{
      expectJS(toolbar.instance()).toIncludeKeys(['inlineToggle', 'blockToggle', 'linkToggle', 'alignmentToggle', 'customBlockToggle']);
    });
  });

  describe('Toolbar inner functions tests', () => {
    let changeStateSpy, linkToggleSpy, toggleAlignmentSpy, editorState, editorComponent;
    beforeEach(() => {
      changeStateSpy = jest.fn();
      toggleAlignmentSpy = jest.fn();
      linkToggleSpy = jest.fn();
      editorState = EditorState.createEmpty();
      editorComponent = shallow(
        <EditorComponent
          changeEditorState={ changeStateSpy }
          toggleAlignment={ toggleAlignmentSpy }
          linkToggle={ linkToggleSpy }
          editorState={ editorState }
          toolbarConfig={ { ...buttons, linkCaptions, configured: true } }
          linkPrompt={ {
            showLinkPrompt: false,
            urlValue: '',
            linkCaption: ''
          } }
          init={ true }
        />
      );
    });
    it('check toolbar blockToggle function calls changeState with relevant state', ()=>{
      editorComponent.find(Toolbar).shallow().instance().blockToggle('ordered-list-item');
      const expectedState = RichUtils.toggleBlockType(editorState, 'ordered-list-item');
      expect(changeStateSpy).toBeCalledWith(expectedState);
    });

    it('check toolbar inlineToggle function calls changeState with relevant state', ()=>{
      editorComponent.find(Toolbar).shallow().instance().inlineToggle('BOLD');
      const expectedState = RichUtils.toggleInlineStyle(editorState, 'BOLD');
      expect(changeStateSpy).toBeCalledWith(expectedState);
    });

    it('check toolbar alignmentToggle function calls changeState without exception', ()=>{
      editorComponent.find(Toolbar).shallow().instance().alignmentToggle('right');
      expect(changeStateSpy).toBeCalled();
    });

    it('check toolbar customBlockToggle changeState is called without exceptions', ()=>{
      editorComponent.find(Toolbar).shallow().instance().customBlockToggle('hr');
      expect(changeStateSpy).toBeCalled();
    });

    it('check toolbar linkToggle on add is called without exceptions', ()=>{
      editorComponent.find(Toolbar).shallow().instance().linkToggle('add');
      expect(linkToggleSpy).toBeCalled();
    });
  });
});