// @flow
import merge from 'lodash/merge';
import Types from '../actions/types';

export const INITIAL_STATE: Object = {
  alignment: 'left',
  lifecycle: 'configuring',
  activeEntityAtCursor: null
};


const setStateKey = (key) => (state, newState) => {
  return { ...state, [key]: newState[key] };
};

const setEditorState = (state :Object, { editorState, activeEntityAtCursor }) => {
  return merge(state, {
    editorState,
    activeEntityAtCursor
  });
};

const setEditor = (state, { editorState }) => {
  return merge(state, {
    editorState,
    linkPrompt: {
      showLinkPrompt: false,
      urlValue: ''
    }
  });
};

const setLinkPrompt = (state, { linkPrompt }) => {
  return {
    ...state,
    linkPrompt: {
      ...state.linkPrompt,
      ...linkPrompt
    }
  };
};

export const handlers = {
  [Types.CONFIGURE_EDITOR]: setEditor,
  [Types.EDITOR_CHANGE_STATE]: setEditorState,
  [Types.LINK_CHANGE_STATE]: setLinkPrompt,
  [Types.MOUNT_EDITOR]: setStateKey('init'),
  [Types.CONFIGURE_EDITOR_API]: setStateKey('subscribers'),
  [Types.UPDATE_LIFECYCLE]: setStateKey('lifecycle'),
  [Types.EDITOR_TRANSFORM_TO_RAW_STATE]: setStateKey('json'),
  [Types.CONFIGURE_TOOLBAR]: setStateKey('toolbarConfig'),
  [Types.UPDATE_WORD_COUNT]: setStateKey('wordCount'),
  [Types.UPDATE_CHAR_COUNT]: setStateKey('charCount'),
  [Types.TOGGLE_ALIGNMENT]: setStateKey('alignment')
};

export default function editorReducer(state :Object = INITIAL_STATE, action :{ type: string }) {
  if (action && action.type) {
    const { type } = action;
    return Object.keys(handlers).includes(type) ? handlers[type](state, action) : state;
  }
  return state;
}

