import expect from 'expect';
import editorReducer, { INITIAL_STATE, handlers }  from './editorReducer';
import Types from '../actions/types';

const runReducerFn = (functionName, payload) => {
  return editorReducer(INITIAL_STATE, { type: functionName, ...payload });
};

describe('editorReducer test', () => {
  it('editorReducer should be a function', ()=>{
    expect(editorReducer).toBeA(Function);
  });
  it('editorReducer should return INITIAL_STATE', ()=>{
    expect(editorReducer()).toEqual(INITIAL_STATE);
  });
  it('editorReducer should contain correct handlers list', ()=>{
    expect(handlers).toIncludeKeys([
      Types.CONFIGURE_EDITOR,
      Types.LINK_CHANGE_STATE,
      Types.MOUNT_EDITOR,
      Types.EDITOR_CHANGE_STATE,
      Types.CONFIGURE_EDITOR_API,
      Types.EDITOR_TRANSFORM_TO_RAW_STATE,
      Types.CONFIGURE_TOOLBAR,
      Types.UPDATE_WORD_COUNT,
      Types.UPDATE_CHAR_COUNT,
      Types.UPDATE_LIFECYCLE
    ]);
  });

  it('CONFIGURE_EDITOR action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.CONFIGURE_EDITOR,
        { plugins: [] }
      ))
      .toIncludeKeys(['alignment', 'linkPrompt']);
  });
  it('LINK_CHANGE_STATE action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.LINK_CHANGE_STATE,
        {
          linkPrompt: {
            showLinkPrompt: true,
            urlValue: ""
          }
        }).linkPrompt.showLinkPrompt)
      .toBeTruthy();
  });
  it('MOUNT_EDITOR action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.MOUNT_EDITOR,
        {
          init: true
        }).init)
      .toBeTruthy();
  });


  it('EDITOR_CHANGE_STATE action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.EDITOR_CHANGE_STATE,
        {
          editorState: {},
          activeEntityAtCursor: 'unstyled'
        }))
      .toIncludeKeys(['editorState', 'activeEntityAtCursor']);
  });

  it('CONFIGURE_EDITOR_API action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.CONFIGURE_EDITOR_API,
        {
          subscribers: {}
        }))
      .toIncludeKeys(['subscribers']);
  });


  it('EDITOR_TRANSFORM_TO_RAW_STATE action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.EDITOR_TRANSFORM_TO_RAW_STATE,
        {
          json: {}
        }))
      .toIncludeKeys(['json']);
  });

  it('CONFIGURE_TOOLBAR action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.CONFIGURE_TOOLBAR,
        {
          toolbarConfig: {}
        }))
      .toIncludeKeys(['toolbarConfig']);
  });
  it('UPDATE_WORD_COUNT action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.UPDATE_WORD_COUNT,
        {
          wordCount: {}
        }))
      .toIncludeKeys(['wordCount']);
  });
  it('UPDATE_CHAR_COUNT action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.UPDATE_CHAR_COUNT,
        {
          charCount: 0
        }))
      .toIncludeKeys(['charCount']);
  });

  it('TOGGLE_ALINGMENT action handler should return correct state', ()=>{
    const reducerResult = runReducerFn(
      Types.TOGGLE_ALIGNMENT,
      {
        alignment: 'right'
      });
    expect(reducerResult).toIncludeKeys(['alignment']);
    expect(reducerResult.alignment).toEqual('right');
  });

  it('UPDATE_LIFECYCLE action handler should return correct state', ()=>{
    expect(
      runReducerFn(
        Types.UPDATE_LIFECYCLE,
        {
          lifecycle: "configured"
        }).lifecycle)
      .toEqual("configured");
  });
});