// @flow
/* eslint-disable no-constant-condition */
import { take, fork } from 'redux-saga/effects';
import { takeEvery } from 'redux-saga';
import Types from '../actions/types';
import type { ReduxStoreType } from '../core/types';

export const stateObserver = (store: ReduxStoreType) => function * ({ json } : Object): Generator<*, void, void> {
  const { editor: { subscribers, lifecycle } } = store.getState();
  if (subscribers && lifecycle === "configured"){
    const { state: observer } = subscribers;
    observer && observer({
      state: {
        ...json
      }
    });
  }
};

export const wordCountObserver = (store: ReduxStoreType) => function * ({ wordCount }: Object): Generator<*, void, void> {
  const { editor: { subscribers, lifecycle } } = store.getState();
  if (subscribers && lifecycle === "configured") {
    const { wordCount: observer } = subscribers;
    observer && observer(wordCount);
  }
};

export const charCountObserver = (store: ReduxStoreType) => function * ({ charCount }: Object): Generator<*, void, void>{
  const { editor: { subscribers, lifecycle } } = store.getState();
  if (subscribers && lifecycle === "configured") {
    const { charCount: observer } = subscribers;
    observer && observer(charCount);
  }
};

export function* apiConfigObserver(store: ReduxStoreType): Generator<*, void, void>{
  const action: any = yield take(Types.GLOBAL_API_CONFIG);
  const { editor: { subscribers } } = store.getState();
  if (subscribers) {
    const { api: observer }  = subscribers;
    observer && observer(action.apiConfig);
  }
}


export const configuredStateObserversTrigger = (store: ReduxStoreType) => function* (): Generator<*, void, void> {
  yield take(Types.UPDATE_LIFECYCLE);
  const { editor } = store.getState();
  const { initialState, initialStateObserversTrigger } = editor;
  if (initialState && initialStateObserversTrigger) {
    initialStateObserversTrigger.map((observer) => {
      editor.subscribers && editor.subscribers[observer](editor[observer]);
    });
  }
};

export default function * apiSaga(store: ReduxStoreType): Generator<*, void, void> {
  yield takeEvery(Types.EDITOR_TRANSFORM_TO_RAW_STATE, stateObserver(store));
  yield takeEvery(Types.UPDATE_WORD_COUNT, wordCountObserver(store));
  yield takeEvery(Types.UPDATE_CHAR_COUNT, charCountObserver(store));
  yield fork(apiConfigObserver, store);
  yield fork(configuredStateObserversTrigger(store));
}