import expectJS from 'expect';
import apiSaga, { stateObserver, wordCountObserver, charCountObserver, apiConfigObserver, configuredStateObserversTrigger } from './apiSaga';
import Types from '../actions/types';
import testSaga from 'redux-saga-test-plan';
import storeGenerator from '../core/storeGenerator';
import storeGetStateMock from '../__mocks__/storeGetStateMock';
import mockedInitialState from '../__mocks__/editorState.json';

describe('Test api saga', ()=>{
    it('apiSaga is generator function', ()=>{
      const GeneratorFunction = function*(){}.constructor;
      expectJS(apiSaga).toBeA(GeneratorFunction);
    });

  it('apiSaga is generator function', ()=>{
    const saga = testSaga(apiSaga);

  });

    it('stateObserver triggers observer', ()=>{
      const store = storeGenerator();
      const stateObserverSpy = jest.fn();
      store.getState = storeGetStateMock({
        state: stateObserverSpy
      });
      const saga = testSaga(stateObserver(store), { json: { test: true } });
      saga.next();
      expect(stateObserverSpy).toBeCalledWith({
        state: {
          test: true
        }
      });
    });

    it('wordCountObserver takes UPDATE_WORD_COUNT and triggers observer when conditions met', ()=>{
      const store = storeGenerator();
      const wordCountObserverSpy = jest.fn();
      store.getState = storeGetStateMock({
        wordCount: wordCountObserverSpy
      });
      const saga = testSaga(wordCountObserver(store), { wordCount: 20 });
      saga.next();
      expect(wordCountObserverSpy).toBeCalledWith(20);
    });

  it('wordCountObserver takes UPDATE_CHAR_COUNT and triggers observer when conditions met', ()=>{
    const store = storeGenerator();
    const charCountObserverSpy = jest.fn();
    store.getState = storeGetStateMock({
      charCount: charCountObserverSpy
    });
    const saga = testSaga(charCountObserver(store), { charCount: 20 });
    saga.next();
    expect(charCountObserverSpy).toBeCalledWith(20);
  });

    it('apiConfigObserver takes GLOBAL_API_CONFIG and triggers observer when conditions met', ()=>{
      const store = storeGenerator();
      const apiConfigSpy = jest.fn();
      store.getState = storeGetStateMock({
        api: apiConfigSpy
      });
      const saga = testSaga(apiConfigObserver, store);
      saga.next().take(Types.GLOBAL_API_CONFIG);
      saga.next({ apiConfig: { test: true } });
      expect(apiConfigSpy).toBeCalledWith({
        test: true
      });
    });

    it('configuredStateObserversTrigger takes UPDATE_LIFECYCLE and triggers observer when conditions met', ()=>{
      const store = storeGenerator();
      const configuredStateObserversSpy = jest.fn();
      store.getState = storeGetStateMock({
        wordCount: configuredStateObserversSpy
      }, {
        initialState: mockedInitialState,
        initialStateObserversTrigger: ['wordCount'],
        wordCount: 0,
        charCount: 5,
      });

      const saga = testSaga(configuredStateObserversTrigger(store));
      saga.next().take(Types.UPDATE_LIFECYCLE);
      saga.next();
      expect(configuredStateObserversSpy).toBeCalledWith(0);
    });
});