// @flow
import { take, put } from 'redux-saga/effects';
import Types from '../actions/types';
import { changeEditorState, configureEditorApi, init, throwConfigurationError, globalApiConfig, updateLifeCycle } from '../actions/creators';
import { EditorState, convertFromRaw, CompositeDecorator } from 'draft-js';

function validateSubscribers(subscribers: Object): { error: boolean, message: string } {
  const invalidSubscribers = Object.keys(subscribers).filter(key => typeof subscribers[key] !== 'function');
  return {
    error: invalidSubscribers.length > 0,
    message: `subscribers must be functions, the following subscribers type is incorrect: ${invalidSubscribers.join(',')}`,
  };
}


export function * watchStartup(): Generator<*, void, void> {
  const action: any = yield take(Types.CONFIGURE_EDITOR);
  const { config, plugins, store } = action;
  const apiConfig = store.getState().editor ? {
    // $FlowFixMe: TODO supressed flow getters as a known flow issue
    get rawState() {
      return store.getState().editor.editorState;
    },
    // $FlowFixMe: TODO supressed flow getters as a known flow issue
    get editorState() {
      return store.getState().editor.json;
    },
    // $FlowFixMe: TODO supressed flow getters as a known flow issue
    get wordCount() {
      return store.getState().editor.wordCount;
    },
    // $FlowFixMe: TODO supressed flow getters as a known flow issue
    get charCount() {
      return store.getState().editor.charCount;
    }
  } : {};
  if (config){
    const { subscribers } = config;
    if (subscribers) {
      const { error, message } = validateSubscribers(subscribers);
      const actionCreator = !error ? configureEditorApi.bind(null, subscribers) : throwConfigurationError.bind(null, message);
      yield put(actionCreator());
    }
  }
  yield put(globalApiConfig(apiConfig));
  yield take([Types.CONFIGURE_TOOLBAR, Types.CONFIGURE_EDITOR_API, Types.GLOBAL_API_CONFIG]);
  if (config) {
    const { initialState } = config;
    if (initialState) {
      yield put(
        changeEditorState(EditorState.createWithContent(
          convertFromRaw(initialState),
          new CompositeDecorator(plugins)
        ), store.getState().json)
      );
      yield take(Types.EDITOR_TRANSFORM_TO_RAW_STATE);
    }
  }
  yield put(
    updateLifeCycle('configured')
  );
  yield put(init());
}