
import { watchStartup } from './startupSaga';
import Types from '../actions/types';
import testSaga from 'redux-saga-test-plan';
import { EditorState, CompositeDecorator } from 'draft-js';
import storeGenerator from '../core/storeGenerator';
import mockedInitialState from '../__mocks__/editorState.json';


function assertGlobalApiConf(saga, store, config = {}, test){
  saga.next({
    type: Types.CONFIGURE_EDITOR,
    plugins: [],
    config,
    store
  }).put(test || {
    type: Types.GLOBAL_API_CONFIG,
    apiConfig: {
      get rawState() {
        return store.getState().editor.editorState;
      },
      get editorState() {
        return store.getState().editor.json;
      },
      get wordCount() {
        return store.getState().editor.wordCount;
      },
      get charCount() {
        return store.getState().editor.charCount;
      }
    }
  });
}

function assertWaitForConfigSagas(saga){
  saga.next().take([Types.CONFIGURE_TOOLBAR, Types.CONFIGURE_EDITOR_API, Types.GLOBAL_API_CONFIG]);
}

function assertUpdateLifecycle(saga){
  saga.next().put({
    type: Types.UPDATE_LIFECYCLE,
    lifecycle: 'configured'
  });
}

function assertMountEditor(saga){
  saga.next().put({
    type: Types.MOUNT_EDITOR,
    init: true
  });
}

describe('(sagas/startupSaga_test.js) - test editor sagas', ()=>{
  let saga;
  let store;
  beforeEach(()=>{
    saga = testSaga(watchStartup);
    store = storeGenerator();
    saga.next().take(Types.CONFIGURE_EDITOR);
  });
  it('watchStartup saga flow is correct without config', ()=>{
    assertGlobalApiConf(saga, store);
    assertWaitForConfigSagas(saga, store);
    assertUpdateLifecycle(saga);
    assertMountEditor(saga);
  });
  it('watchStartup saga flow is correct with config but without subscribers', ()=>{
    assertGlobalApiConf(saga, store, {
      initialState: mockedInitialState
    });
    assertWaitForConfigSagas(saga, store);
    saga.next();
    saga.next().take(Types.EDITOR_TRANSFORM_TO_RAW_STATE);
    assertUpdateLifecycle(saga);
    assertMountEditor(saga);
  });
  it('watchStartup saga flow is correct with config and with subscribers', ()=>{
    const mockedSubscribers = {
      state: ()=>{},
      api: ()=>{},
      wordCount: ()=>{},
      charCount: ()=>{}
    };

    assertGlobalApiConf(saga, store, {
      initialState: mockedInitialState,
      subscribers: mockedSubscribers
    }, {
      type: Types.CONFIGURE_EDITOR_API,
      subscribers: mockedSubscribers
    });
    saga.next();
    assertWaitForConfigSagas(saga, store);
    saga.next();
    saga.next().take(Types.EDITOR_TRANSFORM_TO_RAW_STATE)
    assertUpdateLifecycle(saga);
    assertMountEditor(saga);

  });
});