// @flow
import { take, put } from 'redux-saga/effects';
import Types from '../actions/types';
import { configureToolbar, throwConfigurationError } from '../actions/creators';
import { validateButtonsConfig } from '../utils/toolbarValidator';
import { linkCaptions, buttons as defaultButtonsConfig } from '../config/defaults';
export function * watchToolbarConfig(): Generator<*, void, void> {
  const action: any = yield take(Types.CONFIGURE_EDITOR);
  const { config } = action;
  if (config && config.toolbarConfig){
    const { toolbarConfig: outerConfig } = config;
    const { buttonsConf, buttonsList } = defaultButtonsConfig;
    const validatedButtonsConfig = {
      buttonsList: outerConfig.buttonsList || buttonsList,
      buttonsConf: outerConfig.buttonsConf || buttonsConf,
      linkCaptions: outerConfig.linkCaptions ? { ...linkCaptions, ...outerConfig.linkCaptions } : linkCaptions,
      configured: true
    };
    const errors: Array<string> = validateButtonsConfig(validatedButtonsConfig);
    if (errors.length > 0) {
      yield put(throwConfigurationError(errors.join('\n')));
    } else {
      yield put(configureToolbar(validatedButtonsConfig));
    }
  } else {
    yield put(configureToolbar({ ...defaultButtonsConfig, linkCaptions, configured: true }));
  }
}

