import expect from 'expect';
import 'colors';
import { EditorState } from 'draft-js';
import Types from '../actions/types';
import testSaga from 'redux-saga-test-plan';
import { watchToolbarConfig } from './toolbarSaga';
import { linkCaptions, buttons as defaultButtonsConfig } from '../config/defaults';



describe('(sagas/toolbarSaga_test.js) - test toolbar saga', ()=>{
  describe('Saga structure', ()=>{
    let saga;
    let id;
    beforeEach(()=>{
      saga = testSaga(watchToolbarConfig);
    });
    let generator;
    let GeneratorFunction;
    beforeEach(()=>{
      GeneratorFunction = function*(){}.constructor;
      generator = watchToolbarConfig();
    });


    it('watchToolbarConfig is generator function', ()=>{
      expect(watchToolbarConfig).toBeA(GeneratorFunction);
    });

    it('watchToolbarConfig triggers CONFIGURE_EDITOR action when buttonsConf passed correctly', ()=>{
      saga
        .next()
        .take(Types.CONFIGURE_EDITOR)
        .next({ config: { toolbarConfig: defaultButtonsConfig } })
        .put({
          type: Types.CONFIGURE_TOOLBAR,
          toolbarConfig: { ...defaultButtonsConfig, linkCaptions, configured: true }
        });
    });
    it('watchToolbarConfig triggers CONFIGURE_EDITOR action with defaults when buttonsConf not passed', ()=>{
      saga
        .next()
        .take(Types.CONFIGURE_EDITOR)
        .next({ config: { } })
        .put({
          type: Types.CONFIGURE_TOOLBAR,
          toolbarConfig: { ...defaultButtonsConfig, configured: true, linkCaptions }
        });
    });

    it('watchToolbarConfig triggers CONFIGURATION_ERROR action when buttonsConf passed incorrectly', ()=>{
      saga
        .next()
        .take(Types.CONFIGURE_EDITOR)
        .next({ config: { toolbarConfig: {
          buttonsList: ['C', 'A']
        } } })
        .put({
          type: Types.CONFIGURATION_ERROR,
          message:
            `C is not defined in buttonsConf\nA is not defined in buttonsConf`
        });
    });
  });
});