// @flow
import { ALLOWED_CONTROL_TYPES, ALLOWED_BLOCK_TYPES } from '../config/alowedTypes';
import forOwn from 'lodash/forOwn';

export function assertButtonsListConfig(buttonsList :Array<string>, buttonsConf: Object) :Array<string>{
  return buttonsList
    .filter((key: string) => !buttonsConf[key])
    .map((key :string) => `${key} is not defined in buttonsConf`);
}

export function assertButtonsConfig(errMessageArray: Array<string>, buttonsConf: Object, type: string, allowedTypes: Array<string>) :Array<string>{
  let assertionErrors = errMessageArray;
  forOwn(buttonsConf, block => {
    const blockOrControlType = block[type];
    if (block.controlType === 'menu'){
      assertionErrors = assertButtonsConfig(assertionErrors, block.buttons, type, allowedTypes);
    } else if (!allowedTypes.includes(blockOrControlType)){
      assertionErrors = [ ...assertionErrors, `${blockOrControlType} ${type} is not allowed` ];
    }
  });
  return assertionErrors;
}

export function validateButtonsConfig(toolbarConfig :Object, callback :?Function) :Array<string>{
  const { buttonsConf, buttonsList } = toolbarConfig;
  let errors = [
    ...assertButtonsListConfig(buttonsList, buttonsConf),
    ...assertButtonsConfig([], buttonsConf, 'controlType', ALLOWED_CONTROL_TYPES),
    ...assertButtonsConfig([], buttonsConf, 'blockType', ALLOWED_BLOCK_TYPES)
  ];
  return callback ? callback(errors) : errors;
}
